/*
File: HOOKINT.CC

Author: Matthew Mastracci

Purpose: Hooks the timer interrupt to control the countdown of two independant
timers.  Uses a passive polling loop to test for timer expiry.
*/

#include <stdio.h>
#include <pc.h>
#include <dpmi.h>
#include <go32.h>

/* 
  This macro was used by Shawn Hargreaves in Allegro to lock all of its 
  data: it's great to have around.
*/
#define LOCK_VARIABLE(x)    _go32_dpmi_lock_data((void *)&x, (long)sizeof(x));

extern void TickHandler(void);
extern int TickHandler__Size;

int Timer1 = 1, Timer2 = 1, Flag1, Flag2, TickCount = 0;

int main()
{
  _go32_dpmi_seginfo OldHandler, NewHandler;

  /* Ensure we can get through the code without GPFing the system */
  printf("Grabbing timer interrupt...\n");
  _go32_dpmi_lock_code(TickHandler, (long)TickHandler__Size);
  LOCK_VARIABLE(Timer1);
  LOCK_VARIABLE(Timer2);
  LOCK_VARIABLE(Flag1);
  LOCK_VARIABLE(Flag2);
  LOCK_VARIABLE(TickCount);
  
  /* Store the old vector so we can clean up later */
  _go32_dpmi_get_protected_mode_interrupt_vector(8, &OldHandler);
  
  /* Start the handler going */
  NewHandler.pm_offset = (int)TickHandler;
  NewHandler.pm_selector = _go32_my_cs();
  _go32_dpmi_chain_protected_mode_interrupt_vector(8, &NewHandler);

  /* Passive timer polling loop */
  while (!kbhit())
  {
    if (Flag1) 
    {
      printf("Timer 1 expired: %i\n", TickCount);
      Flag1 = 0;
      Timer1 = 5;
    }
    if (Flag2) 
    {
      printf("Timer 2 expired: %i\n", TickCount);
      Flag2 = 0;
      Timer2 = 7;
    }
  }
  getkey();

  /* Clean up */
  printf("Releasing timer interrupt...\n");
  _go32_dpmi_set_protected_mode_interrupt_vector(8, &OldHandler);

  return 0;
}
